/*
 * This is CacheGo, copyright Evan Daniel 2002.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation - version 2.     
 *                                                             
 * This program is distributed in the hope that it will be       
 * useful, but WITHOUT ANY WARRANTY; without even the implied    
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       
 * PURPOSE.  See the GNU General Public License in file COPYING  
 * for more details.                                             
                                                              
 * You should have received a copy of the GNU General Public     
 * License along with this program; if not, write to the Free    
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       
 * Boston, MA 02111, USA.                                        
 *
 */

/*
 * This is the main engine portion, which actually attempts
 * to play Go.  It has a Board and a Cache.
 *
 * All AI function calls should be passed through the cache.
 *
 * Currently, however, it just contains enough code to prove
 * the cache scheme works.  We do some addition.  Exciting.
 *
 * The Cache and the Engine know about each other -- when
 * the Engine wants to make a function call, it probably
 * wants a cached result if possible.  So, it calls the
 * function through the Cache, which checks for an available
 * result and returns it.
 *
 * Note that any function which might be cached must make *all*
 * of its calls through the cache (those that affect output, anyway)
 * even if it does not want a cached result.  This is so that
 * the automatic dependency code does not return a cached result
 * when not applicable because it is unaware of the correct
 * dependencies.
 */

class Engine
{
	Cache myCache;
	int a;
	int b;
	int c;

	Engine()
	{
		myCache = new Cache(this);
	}

	public void setA(int n) {a = n;}
	public void setB(int n) {b = n;}
	public void setC(int n) {c = n;}
	public int getA() {return a;}
	public int getB() {return b;}
	public int getC() {return c;}
	

	public int sum()
	{
		return ((Integer)(myCache.call("getA", new Object[0]))).intValue() +
				((Integer)(myCache.call("getB", new Object[0]))).intValue();
	}

	public int sum3()
	{
		return ((Integer)(myCache.call("sum", new Object[0]))).intValue() +
				((Integer)(myCache.call("getC", new Object[0]))).intValue();
	}
}
