/*
 * This is CacheGo, copyright Evan Daniel 2002.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation - version 2.     
 *                                                             
 * This program is distributed in the hope that it will be       
 * useful, but WITHOUT ANY WARRANTY; without even the implied    
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       
 * PURPOSE.  See the GNU General Public License in file COPYING  
 * for more details.                                             
                                                              
 * You should have received a copy of the GNU General Public     
 * License along with this program; if not, write to the Free    
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       
 * Boston, MA 02111, USA.                                        
 *
 */

/* 
 * This is the object to store function calls that have been made
 * through the cache.  Basically, it has a pointer to the function
 * to be called, and a list of the arguments to that function.
 *
 * Calling the invoke method returns the same object as calling the
 * function directly, with the distinction that a typecast must be
 * made to get the right final type.
 */
import java.lang.reflect.*;

class FunctionCall
{
	public Method function;
	public Object[] args;

	FunctionCall(Method m, Object[] a)
	{
		function = m;
		args = a;
	}

	public Object invoke(Engine eng)
	{
		try
		{
			return function.invoke(eng, args);
		} catch (Exception e)
		{
			System.err.println("Error invoking function " + function.getName());
			System.err.println(e.toString());
		}
		return null;
	}

	public boolean equals(Object obj)
	{
		if (!(obj instanceof FunctionCall))
			return false;
		FunctionCall fCall = (FunctionCall)(obj);
		if (!this.function.equals(fCall.function)) return false;
		if (this.args.length != fCall.args.length) return false;
		boolean eq = true;
		for (int i = 0; i < args.length; i++)
			eq = eq && args[i] == fCall.args[i];
		return eq;
	}

	public int hashCode()
	{
		int h = function.hashCode();
		for (int i = 0; i < args.length; i++)
			h += args[i].hashCode();
		return h;
	}
}
