/*
 * This is CacheGo, copyright Evan Daniel 2002.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation - version 2.     
 *                                                             
 * This program is distributed in the hope that it will be       
 * useful, but WITHOUT ANY WARRANTY; without even the implied    
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       
 * PURPOSE.  See the GNU General Public License in file COPYING  
 * for more details.                                             
                                                              
 * You should have received a copy of the GNU General Public     
 * License along with this program; if not, write to the Free    
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       
 * Boston, MA 02111, USA.                                        
 *
 */

/* 
 * This is the class for the result of a cached function.
 * It stores the return value of the function, as well as all
 * dependencies needed to guarantee the correctness of that
 * result.  Currently, dependencies are FunctionCalls, not
 * FunctionResults.  The cache is responsible for checking
 * whether the result of the Call is the same as last time,
 * and therefore the one we want.
 */
import java.util.Vector;
import java.lang.reflect.*;


class FunctionResult
{
	public Object retval;
	public Vector deps;
	private FunctionResult oldDepPtr;

	FunctionResult(Engine e, Cache c, FunctionCall fCall)
	{
		deps = new Vector();
		oldDepPtr = c.depPtr;
		c.depPtr = this;
		retval = fCall.invoke(e);
		c.depPtr = oldDepPtr;
		oldDepPtr = null;
	}

	public void addDep(FunctionCall fCall)
	{
		deps.addElement(fCall);
	}

	public boolean equals(Object obj)
	{
		if (!(obj instanceof FunctionResult)) return false;
		FunctionResult f = ((FunctionResult)(obj));
		return this.retval.equals(f.retval) && this.deps.equals(f.deps);
	}

	public int hashCode()
	{
		return retval.hashCode() + deps.hashCode();
	}
}
